// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/event/room/state/state_event.dart';
import 'package:meta/meta.dart';

const _roomAvatarType = 'm.room.avatar';

class AvatarChangeEvent extends StateEvent {
  static const String type = _roomAvatarType;

  @override
  final String matrixType = type;

  @override
  final AvatarChange content;

  AvatarChangeEvent(
      this.content, RoomEventArgs args)
      : super(args, stateKey: '', );

  factory AvatarChangeEvent.fromJson(RoomEventArgs args, dynamic json) {
    var url = json['content']['url'];

    // TODO: Handle avatar removals
    if (url == null) {
      return null;
    }

    return AvatarChangeEvent(
      AvatarChange(url: Uri.parse(url)), args);
  }
}

class AvatarChange extends StateEventContent {
  @override
  final String matrixType = _roomAvatarType;

  /// An `mxc` url pointing to the avatar.
  final Uri url;

  AvatarChange({@required this.url});

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'url': url.toString()
    });
}