// Copyright (C) 2019  Wilko Manger
// Copyright (C) 2019  Mathieu Velten
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/homeserver.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/user.dart';

/// Stores all data (rooms, users, events) somewhere.
abstract class Store {

  bool get isOpen;

  Future<bool> open({LocalUser user});
  Future<void> close();

  Future<void> delete();

  Future<UserId> getUserId();
  Future<void> setUserId(UserId value);

  Future<String> getAccessToken();
  Future<void> setAccessToken(String value);

  Future<String> getSyncToken();
  Future<void> setSyncToken(String value);

  Future<Homeserver> getHomeserver();
  Future<void> setHomeserver(Homeserver value);

  Future<JoinedRoom> getJoinedRoom(RoomId id);
  Future<void> setJoinedRooms(Iterable<JoinedRoom> rooms);
  Stream<JoinedRoom> getJoinedRooms();

  Future<RoomEvent> getRoomEvent(Room room, EventId id);
  Future<void> removeEvent(EventId id);
  Stream<RoomEvent> getRoomEvents(Room room, {int count, RoomEvent from});
  Stream<RoomEvent> getUnsentEvents(Room room);
  Future<void> addRoomEvents(Room room, Iterable<RoomEvent> events);

  Future<User> getMember(RoomId roomId, UserId id, {DateTime at});
  Stream<User> getMembers(RoomId id);
  Future<void> addMembers(RoomId roomId, Iterable<User> members);

  Future<void> setAppCustomData(String key, String value);
  Future<String> getAppCustomData(String key);

  Future<void> setObjectCustomData(Id objectId, String key, String value);
  Future<String> getObjectCustomData(Id objectId, String key);
}
